@@ -7,9 +7,10 @@ def start_webserver(args: Dict[str, Any]) -> None:
     """
     Main entry point for webserver mode
     """
-    from freqtrade.configuration import Configuration
+    from freqtrade.configuration import setup_utils_configuration
     from freqtrade.rpc.api_server import ApiServer
 
     # Initialize configuration
-    config = Configuration(args, RunMode.WEBSERVER).get_config()
+
+    config = setup_utils_configuration(args, RunMode.WEBSERVER)
     ApiServer(config, standalone=True)
