@@ -4,7 +4,7 @@ Volume PairList provider
 Provides dynamic pair list based on trade volumes
 """
 import logging
-from datetime import datetime, timedelta, timezone
+from datetime import timedelta
 from typing import Any, Dict, List, Literal
 
 from cachetools import TTLCache
@@ -13,8 +13,8 @@ from freqtrade.constants import Config, ListPairsWithTimeframes
 from freqtrade.exceptions import OperationalException
 from freqtrade.exchange import timeframe_to_minutes, timeframe_to_prev_date
 from freqtrade.exchange.types import Tickers
-from freqtrade.misc import format_ms_time
-from freqtrade.plugins.pairlist.IPairList import IPairList
+from freqtrade.plugins.pairlist.IPairList import IPairList, PairlistParameter
+from freqtrade.util import dt_now, format_ms_time
 
 
 logger = logging.getLogger(__name__)
@@ -25,6 +25,8 @@ SORT_VALUES = ['quoteVolume']
 
 class VolumePairList(IPairList):
 
+    is_pairlist_generator = True
+
     def __init__(self, exchange, pairlistmanager,
                  config: Config, pairlistconfig: Dict[str, Any],
                  pairlist_pos: int) -> None:
@@ -60,16 +62,16 @@ class VolumePairList(IPairList):
 
         # get timeframe in minutes and seconds
         self._tf_in_min = timeframe_to_minutes(self._lookback_timeframe)
-        self._tf_in_sec = self._tf_in_min * 60
+        _tf_in_sec = self._tf_in_min * 60
 
         # wether to use range lookback or not
         self._use_range = (self._tf_in_min > 0) & (self._lookback_period > 0)
 
-        if self._use_range & (self._refresh_period < self._tf_in_sec):
+        if self._use_range & (self._refresh_period < _tf_in_sec):
             raise OperationalException(
                 f'Refresh period of {self._refresh_period} seconds is smaller than one '
                 f'timeframe of {self._lookback_timeframe}. Please adjust refresh_period '
-                f'to at least {self._tf_in_sec} and restart the bot.'
+                f'to at least {_tf_in_sec} and restart the bot.'
             )
 
         if (not self._use_range and not (
@@ -111,6 +113,53 @@ class VolumePairList(IPairList):
         """
         return f"{self.name} - top {self._pairlistconfig['number_assets']} volume pairs."
 
+    @staticmethod
+    def description() -> str:
+        return "Provides dynamic pair list based on trade volumes."
+
+    @staticmethod
+    def available_parameters() -> Dict[str, PairlistParameter]:
+        return {
+            "number_assets": {
+                "type": "number",
+                "default": 30,
+                "description": "Number of assets",
+                "help": "Number of assets to use from the pairlist",
+            },
+            "sort_key": {
+                "type": "option",
+                "default": "quoteVolume",
+                "options": SORT_VALUES,
+                "description": "Sort key",
+                "help": "Sort key to use for sorting the pairlist.",
+            },
+            "min_value": {
+                "type": "number",
+                "default": 0,
+                "description": "Minimum value",
+                "help": "Minimum value to use for filtering the pairlist.",
+            },
+            **IPairList.refresh_period_parameter(),
+            "lookback_days": {
+                "type": "number",
+                "default": 0,
+                "description": "Lookback Days",
+                "help": "Number of days to look back at.",
+            },
+            "lookback_timeframe": {
+                "type": "string",
+                "default": "",
+                "description": "Lookback Timeframe",
+                "help": "Timeframe to use for lookback.",
+            },
+            "lookback_period": {
+                "type": "number",
+                "default": 0,
+                "description": "Lookback Period",
+                "help": "Number of periods to look back at.",
+            },
+        }
+
     def gen_pairlist(self, tickers: Tickers) -> List[str]:
         """
         Generate the pairlist
@@ -161,13 +210,13 @@ class VolumePairList(IPairList):
             # get lookback period in ms, for exchange ohlcv fetch
             since_ms = int(timeframe_to_prev_date(
                 self._lookback_timeframe,
-                datetime.now(timezone.utc) + timedelta(
+                dt_now() + timedelta(
                     minutes=-(self._lookback_period * self._tf_in_min) - self._tf_in_min)
                     ).timestamp()) * 1000
 
             to_ms = int(timeframe_to_prev_date(
                             self._lookback_timeframe,
-                            datetime.now(timezone.utc) - timedelta(minutes=self._tf_in_min)
+                            dt_now() - timedelta(minutes=self._tf_in_min)
                             ).timestamp()) * 1000
 
             # todo: utc date output for starting date
@@ -211,6 +260,7 @@ class VolumePairList(IPairList):
                     quoteVolume = (pair_candles['quoteVolume']
                                    .rolling(self._lookback_period)
                                    .sum()
+                                   .fillna(0)
                                    .iloc[-1])
 
                     # replace quoteVolume with range quoteVolume sum calculated above
