@@ -218,6 +218,12 @@ class StrategyResolver(IResolver):
                     "Please update your strategy to implement "
                     "`populate_indicators`, `populate_entry_trend` and `populate_exit_trend` "
                     "with the metadata argument. ")
+
+        has_after_fill = ('after_fill' in getfullargspec(strategy.custom_stoploss).args
+                          and check_override(strategy, IStrategy, 'custom_stoploss'))
+        if has_after_fill:
+            strategy._ft_stop_uses_after_fill = True
+
         return strategy
 
     @staticmethod
