@@ -34,6 +34,7 @@ class Webhook(RPCHandler):
         self._format = self._config['webhook'].get('format', 'form')
         self._retries = self._config['webhook'].get('retries', 0)
         self._retry_delay = self._config['webhook'].get('retry_delay', 0.1)
+        self._timeout = self._config['webhook'].get('timeout', 10)
 
     def cleanup(self) -> None:
         """
@@ -44,8 +45,11 @@ class Webhook(RPCHandler):
 
     def _get_value_dict(self, msg: RPCSendMsg) -> Optional[Dict[str, Any]]:
         whconfig = self._config['webhook']
+        if msg['type'].value in whconfig:
+            # Explicit types should have priority
+            valuedict = whconfig.get(msg['type'].value)
         # Deprecated 2022.10 - only keep generic method.
-        if msg['type'] in [RPCMessageType.ENTRY]:
+        elif msg['type'] in [RPCMessageType.ENTRY]:
             valuedict = whconfig.get('webhookentry')
         elif msg['type'] in [RPCMessageType.ENTRY_CANCEL]:
             valuedict = whconfig.get('webhookentrycancel')
@@ -62,9 +66,6 @@ class Webhook(RPCHandler):
                              RPCMessageType.EXCEPTION,
                              RPCMessageType.WARNING):
             valuedict = whconfig.get('webhookstatus')
-        elif msg['type'].value in whconfig:
-            # Allow all types ...
-            valuedict = whconfig.get(msg['type'].value)
         elif msg['type'] in (
                 RPCMessageType.PROTECTION_TRIGGER,
                 RPCMessageType.PROTECTION_TRIGGER_GLOBAL,
@@ -83,7 +84,7 @@ class Webhook(RPCHandler):
             valuedict = self._get_value_dict(msg)
 
             if not valuedict:
-                logger.info("Message type '%s' not configured for webhooks", msg['type'])
+                logger.debug("Message type '%s' not configured for webhooks", msg['type'])
                 return
 
             payload = {key: value.format(**msg) for (key, value) in valuedict.items()}
@@ -107,12 +108,13 @@ class Webhook(RPCHandler):
 
             try:
                 if self._format == 'form':
-                    response = post(self._url, data=payload)
+                    response = post(self._url, data=payload, timeout=self._timeout)
                 elif self._format == 'json':
-                    response = post(self._url, json=payload)
+                    response = post(self._url, json=payload, timeout=self._timeout)
                 elif self._format == 'raw':
                     response = post(self._url, data=payload['data'],
-                                    headers={'Content-Type': 'text/plain'})
+                                    headers={'Content-Type': 'text/plain'},
+                                    timeout=self._timeout)
                 else:
                     raise NotImplementedError(f'Unknown format: {self._format}')
 
