@@ -36,7 +36,7 @@ def merge_informative_pair(dataframe: pd.DataFrame, informative: pd.DataFrame,
     :return: Merged dataframe
     :raise: ValueError if the secondary timeframe is shorter than the dataframe timeframe
     """
-
+    informative = informative.copy()
     minutes_inf = timeframe_to_minutes(timeframe_inf)
     minutes = timeframe_to_minutes(timeframe)
     if minutes == minutes_inf:
@@ -45,10 +45,19 @@ def merge_informative_pair(dataframe: pd.DataFrame, informative: pd.DataFrame,
     elif minutes < minutes_inf:
         # Subtract "small" timeframe so merging is not delayed by 1 small candle
         # Detailed explanation in https://github.com/freqtrade/freqtrade/issues/4073
-        informative['date_merge'] = (
-            informative[date_column] + pd.to_timedelta(minutes_inf, 'm') -
-            pd.to_timedelta(minutes, 'm')
-        )
+        if not informative.empty:
+            if timeframe_inf == '1M':
+                informative['date_merge'] = (
+                    (informative[date_column] + pd.offsets.MonthBegin(1))
+                    - pd.to_timedelta(minutes, 'm')
+                )
+            else:
+                informative['date_merge'] = (
+                    informative[date_column] + pd.to_timedelta(minutes_inf, 'm') -
+                    pd.to_timedelta(minutes, 'm')
+                )
+        else:
+            informative['date_merge'] = informative[date_column]
     else:
         raise ValueError("Tried to merge a faster timeframe to a slower timeframe."
                          "This would create new rows, and can throw off your regular indicators.")
@@ -77,9 +86,6 @@ def merge_informative_pair(dataframe: pd.DataFrame, informative: pd.DataFrame,
                              right_on=date_merge, how='left')
     dataframe = dataframe.drop(date_merge, axis=1)
 
-    # if ffill:
-    #     dataframe = dataframe.ffill()
-
     return dataframe
 
 
@@ -123,7 +129,8 @@ def stoploss_from_open(
     return max(stoploss * leverage, 0.0)
 
 
-def stoploss_from_absolute(stop_rate: float, current_rate: float, is_short: bool = False) -> float:
+def stoploss_from_absolute(stop_rate: float, current_rate: float, is_short: bool = False,
+                           leverage: float = 1.0) -> float:
     """
     Given current price and desired stop price, return a stop loss value that is relative to current
     price.
@@ -136,6 +143,7 @@ def stoploss_from_absolute(stop_rate: float, current_rate: float, is_short: bool
     :param stop_rate: Stop loss price.
     :param current_rate: Current asset price.
     :param is_short: When true, perform the calculation for short instead of long
+    :param leverage: Leverage to use for the calculation
     :return: Positive stop loss value relative to current price
     """
 
@@ -150,4 +158,4 @@ def stoploss_from_absolute(stop_rate: float, current_rate: float, is_short: bool
     # negative stoploss values indicate the requested stop price is higher/lower
     # (long/short) than the current price
     # shorts can yield stoploss values higher than 1, so limit that as well
-    return max(min(stoploss, 1.0), 0.0)
+    return max(min(stoploss, 1.0), 0.0) * leverage
